package com.gree.mall.manager.logic.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.admin.AdminWebsitTree;
import com.gree.mall.manager.bean.admin.AdminWebsitVO;
import com.gree.mall.manager.commonmapper.AdminMapper;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.AdminCompanyWechat;
import com.gree.mall.manager.plus.entity.AdminCompanyWechatPayConfig;
import com.gree.mall.manager.plus.entity.AdminWebsit;
import com.gree.mall.manager.plus.entity.AdminZone;
import com.gree.mall.manager.plus.service.AdminCompanyWechatPayConfigService;
import com.gree.mall.manager.plus.service.AdminCompanyWechatService;
import com.gree.mall.manager.plus.service.AdminWebsitService;
import com.gree.mall.manager.plus.service.AdminZoneService;
import com.gree.mall.manager.utils.StringUtil;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AdminWebsitLogic {

    @Autowired
    AdminWebsitService adminWebsitService;
    @Autowired
    AdminZoneService adminZoneService;
    @Autowired
    CommonLogic commonLogic;

    @Autowired
    AdminMapper adminMapper;

    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;

    @Autowired
    AdminCompanyWechatPayConfigService adminCompanyWechatPayConfigService;

    /**
     * 区域列表
     */
    public List<AdminZone> adminZoneList(){
        List<AdminZone> list = adminZoneService.list();
        return list;
    }

    /**
     * 部门树
     */
    public List<AdminWebsitTree> tree(HttpServletRequest request){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        List<AdminWebsit> list = adminWebsitService.lambdaQuery()
//                .in(CollectionUtils.isNotEmpty(adminUser.getCorpIds()),AdminWebsit::getCorpId,adminUser.getCorpIds())
                .list();
        List<AdminWebsitTree> trees = new ArrayList<>();
        for(AdminWebsit adminWebsit : list){
            AdminWebsitTree adminWebsitTree = new AdminWebsitTree();
            BeanUtils.copyProperties(adminWebsit,adminWebsitTree);
            trees.add(adminWebsitTree);
        }

        List<AdminWebsitTree> parentList = trees.stream().filter(x -> x.getParentId() == 0).collect(Collectors.toList());
        for(AdminWebsitTree adminWebsit:parentList){
            adminWebsit.setChildren(this.treeModule(trees,adminWebsit.getWebsitId(),adminWebsit.getCompanyWechatId()));
        }
        return parentList;
    }


    /**
     *  新增部门
     */
    @Transactional
    public void add(AdminWebsit adminWebsit, HttpServletRequest request) throws WxErrorException, RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getWebsitNumber,adminWebsit.getWebsitNumber())
                .or()
                .eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId() == 0){
            throw new RemoteServiceException("请选择父部门");
        }
        if (StringUtil.isEmpty(adminWebsit.getCompanyWechatId()))
            throw new RemoteServiceException("上级商家不能为空");
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(adminWebsit.getPayWorkerCodeId());
        AdminCompanyWechat adminCompanyWechatP = adminCompanyWechatService.getById(adminWebsit.getCompanyWechatId());

        //Long aLong = workWechatLogic.addOrganization(adminWebsit.getName(), adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());
        String aLong = IdWorker.getIdStr();
        adminWebsit.setWebsitId(aLong+"");
//        adminWebsit.setCorpId(adminUser.getCorpIds().get(0));
        adminWebsit.setWebsitId(aLong);
        adminWebsit.setCompanyName(adminCompanyWechatP.getCompanyName());
        adminWebsit.setPayWorkerCodeName(adminCompanyWechat.getCompanyName());
        adminWebsit.insert();

    }
    /**
     * 编辑部门
     */
    @Transactional
    public void update(AdminWebsit adminWebsit,HttpServletRequest request) throws RemoteServiceException, WxErrorException {

        //AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .ne(AdminWebsit::getWebsitId,adminWebsit.getWebsitId())
                .and(v -> v.eq(AdminWebsit::getWebsitNumber,adminWebsit.getWebsitNumber())
                        .or()
                        .eq(AdminWebsit::getName, adminWebsit.getName())
                )
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId() == 0){
            throw new RemoteServiceException("请选择父部门");

        }

        if (StringUtil.isEmpty(adminWebsit.getCompanyWechatId()))
            throw new RemoteServiceException("上级商家不能为空");
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(adminWebsit.getPayWorkerCodeId());
        AdminCompanyWechat adminCompanyWechatP = adminCompanyWechatService.getById(adminWebsit.getCompanyWechatId());
        adminWebsit.setCompanyName(adminCompanyWechatP.getCompanyName());
        adminWebsit.setPayWorkerCodeName(adminCompanyWechat.getCompanyName());
        adminWebsit.updateById();
        //workWechatLogic.updateOrganization(adminWebsit.getWebsitId(),adminWebsit.getName(),adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());

    }

    /**
     * 删除部门
     */
    @Transactional
    public void delete(String id,HttpServletRequest request) throws WxErrorException {
//s        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        //递归删除
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(this.queryAllChild(list,id));
        List<String> ids = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        ids.add(id);
        if(ids.size() > 0) {
//            for(Long idd : ids){
//                workWechatLogic.deleteOrganization(idd,adminUser.getAdminCompanyWechat());
//            }
            adminWebsitService.lambdaUpdate().in(AdminWebsit::getWebsitId, ids).remove();
        }
    }

    /**
     * 详情
     * @param id
     */
    public AdminWebsit detail(String id){
        AdminWebsit adminWebsit = adminWebsitService.getById(id);
        return adminWebsit;
    }

    /**
     * 详情
     */
    public AdminWebsit detail(Long websitId,String corpId){
        AdminWebsit adminWebsit = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getWebsitId,websitId)
                .one();
        return adminWebsit;
    }

    /**
     * 递归查询所有数据
     */
    public List<AdminWebsit> queryAllChild(List<AdminWebsit> list,String id){
        if(id == null || id.equals("0")){
            return list;
        }
        List<AdminWebsit> adminWebsits = adminWebsitService.lambdaQuery().eq(AdminWebsit::getParentId, id).list();
        if(adminWebsits.size() == 0){
            return list;
        }
        for(AdminWebsit adminWebsit:adminWebsits){
            this.queryAllChild(list,adminWebsit.getWebsitId());
        }
        list.addAll(adminWebsits);
        return list;
    }

    /**
     * 递归组装数据
     * @param list
     * @param parentId
     * @return
     */
    private List<AdminWebsitTree> treeModule(List<AdminWebsitTree> list, String parentId,String companyId){
        //最父级资源树
        if(parentId == null || parentId.equals("0")) {
            return null;
        }
        List<AdminWebsitTree> collect = list.stream()
                .filter(e -> e.getParentId().equals(parentId) && e.getCompanyWechatId().equals(companyId)).collect(Collectors.toList());
        if(collect.size() == 0){
            return null;
        }
        for(AdminWebsitTree bean : collect){
            bean.setChildren(this.treeModule(list, bean.getWebsitId(),bean.getCompanyWechatId()));
        }
        return collect;
    }


    public IPage<AdminWebsitVO> list(Page page, ZfireParamBean zfireParam) {

        //获取当前登录企业id
        AdminUserCom adminUser = commonLogic.getAdminUser();

        //1.组装查询条件
        FieldUtils.supplyParam(zfireParam, AdminWebsitVO.class,adminUser);

        IPage<AdminWebsitVO> adminDeptWebsitVOIPage = adminMapper.listWebsit(page, zfireParam);
        return adminDeptWebsitVOIPage;
    }

    public void importData(List<Object> objects) {

        List<AdminWebsit> adminWebsits = new ArrayList<>();

        for (int i = 0; i < objects.size(); i++) {
            int rowIndex = i + 2;
            List<Object> row = (List<Object>) objects.get(i);

            AdminWebsit adminWebsit = new AdminWebsit();

            adminWebsit.setWebsitId((String) row.get(0));
            adminWebsit.setWebsitNumber((String) row.get(0));
            adminWebsit.setName((String) row.get(1));
            adminWebsit.setMobile((String) row.get(2));
            adminWebsit.setPerson((String) row.get(3));
            adminWebsit.setPersonMobile((String) row.get(4));
            adminWebsit.setType((String) row.get(5));
            adminWebsit.setAddress((String) row.get(6));
            AdminCompanyWechatPayConfig one = adminCompanyWechatPayConfigService.lambdaQuery()
                    .eq(AdminCompanyWechatPayConfig::getCompanyName, (String) row.get(7))

                    .last("limit 1").one();

            if (one == null)
                throw new RemoteServiceException("第"+rowIndex+"行，找不到支付商户");
            adminWebsit.setPayWorkerCodeName((String) row.get(7));
            adminWebsit.setPayWorkerCodeId(one.getId());
            if (row.size() > 7) {
                adminWebsit.setYunAppid((String) row.get(8));
                adminWebsit.setYunAppkey((String) row.get(9));
                adminWebsit.setYunCompany((String) row.get(10));
                adminWebsit.setYunNumber((String) row.get(11));
                adminWebsit.setYunSystem((String) row.get(12));
                adminWebsit.setYunTax((String) row.get(13));
                adminWebsit.setYunName((String) row.get(14));
                adminWebsit.setYunTwo((((String) row.get(15)).equals("是"))?"YES":"NO");
            }

            adminWebsits.add(adminWebsit);

        }

        adminWebsitService.saveBatch(adminWebsits);
    }
}
